'' ILI9325 driver using pins P0-P11 for the display and P16-P23 for the touchscreen
'' James Moxham 2011

CON


  _clkmode      = xtal1 + pll16x                        ' use crystal x 16
  _xinfreq      = 5_000_000

  _cardDataOutPin = 12                                  ' SD card pins
  _cardClockPin = 13
  _cardDataInPin = 14
  _cardChipSelectPin = 15

  _clockDataPin = 29                                    ' clock not used but need to include this for Kye's driver
  _clockClockPin = 28

  _TV_Enabled = true                                    ' TV display on
  _TV_Pin = 16                                          ' group of 3 pins starting at pin 16 = 16,17,18

  Touch_Clock = 23                                        ' Touch screen pins (shared with VGA and one, penirq, shared with TV)
  Touch_ChipSelect = 22
  Touch_DataIn = 21
  Touch_Busy = 20
  Touch_DataOut = 19
  Touch_Penirq = 18
  
OBJ
 delay:         "timing"                                ' thanks to parallax
 fat:           "SD2.0_FATEngine.spin"                  ' thanks to Kye
 text:          "tv_text"                               ' thanks to Chip
 str:           "ASCII0_STREngine.spin"                 ' thanks to Kye 
 spi:           "SPI_ASM"                               ' thanks to Beau
 ILI9325:       "ILI9325_Dracblade"                     ' J Moxham

VAR
                                      
  
PUB Main
    text.start(_TV_Pin)                                 ' start the TV driver
    text.clearscreen                                    ' clear the screen
    ILI9325.start_ram                                   ' start the external ram driver in a cog
    ILI9325.start_ILI9325                               ' start the ILI9325 display
    ILI9325.docmd("H",0,0,0)                            ' set high latch to 000 = first 64k block     
    PrintStringCR(string("Testing for SD card"))        ' helpful message if card is out, better than just a blank screen  
    'Touchscreen                                        ' test the touchscreen
    ifnot(fat.FATEngineStart(_cardDataOutPin, _cardClockPin, _cardDataInPin, _cardChipSelectPin, _clockDataPin, _clockClockPin))
      reboot
    fat.mountPartition(0,0)                             ' mount the sd card
    
    'SlowSDtoMemory                                          ' send to memory then memory to screen
    'MemoryToDisplay                                     ' pasm move memory to display
    'MemoryToDisplay
    'SDtoExternalMemoryBlock
    ExtRamSpeedTest
    
    SDtoHubMemoryBlock

    IconsToMemory
    'ReadIcon                                            ' display a 64x64 icon
    'SlowBackground                                         ' display background picture
    Touchscreen



'PUB ReadIcon
'    fat.openfile(string("photo64.ili"),"R")            ' open an image (.ili extension)
'    ILI9325.draw(0,0,63,63)                           ' set up the area of the screen to draw in
'    repeat 4096
'       ILI9325.pixel(fat.readbyte,fat.readbyte)         ' high byte stored first
'    fat.closefile 

PUB SlowBackground                                          ' slow display to screen, see SDtoMemoryBlock for a much faster version
    PrintStringCR(string("Opening file"))               ' message on the TV screen
    fat.openfile(string("icons320.ili"),"R")            ' open an image (.ili extension)
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in
    repeat 76800
       ILI9325.pixel(fat.readbyte,fat.readbyte)         ' high byte stored first
    fat.closefile   
    PrintStringCR(string("Closing file"))

PUB SlowSDtoMemory | i,j, ramaddress
  ramaddress := 0
  PrintStringCR(string("start writing icon to ram"))  
  fat.openfile(string("photo64.ili"),"R")               ' open an image (.ili extension) 
  repeat 8192                                           ' 4096 * 2 bytes per pixel
    i := fat.readbyte
    ILI9325.docmd("W",@i,ramaddress,1)                      ' write n to ramaddress
    ramaddress := ramaddress + 1                        ' increment the counter
  fat.closefile
  PrintStringCR(string("finished writing to ram"))
  ' now send this to the screen
  ILI9325.draw(64,64,127,127)                           ' set up the area of the screen to draw in  
  ramaddress := 0                                       ' reset the ramaddress
  repeat 4096
    ILI9325.docmd("R",@i,ramaddress,1)                      ' read a byte
    ramaddress := ramaddress + 1                        ' increment counter
    ILI9325.docmd("R",@j,ramaddress,1)                       ' read a byte
    ramaddress := ramaddress + 1                        ' increment counter
    ILI9325.pixel(i,j)                                  ' display

PUB SDtoExternalMemoryBlock                                     ' block move data from sd card to memory, much faster than one byte at a time
    PrintStringCR(string("Block load"))                 ' message on the TV screen
    fat.openfile(string("icons320.ili"),"R")            ' open an image (.ili extension)
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in
    repeat 300
      fat.readdata(@sdbuffer,512)
      ILI9325.docmd("W",@sdbuffer,0,512)                  ' now move data from hub to external ram     
      ILI9325.docmd("I",0,0,512)                          ' move 512 bytes = 256 pixels from external ram to the display
    fat.closefile   
    PrintStringCR(string("Closing file"))

PUB SDtoHubMemoryBlock                                 ' block move data from sd card to hub then block move to display 
    PrintStringCR(string("Block load"))                 ' message on the TV screen
    fat.openfile(string("icons320.ili"),"R")            ' open an image (.ili extension)
    ILI9325.draw(0,0,239,319)                           ' set up the area of the screen to draw in
    repeat 300
      fat.readdata(@sdbuffer,512)                        ' read 512 bytes from sd card to hub ram
      ILI9325.docmd("D",@sdbuffer,0,512)                 ' move 512 bytes = 256 pixels from external ram to the display
    fat.closefile   
    PrintStringCR(string("Closing file"))

PUB ExtRamSpeedTest | ramaddress,x,y                      ' read an icon into external ram then copy multiple times to the screen
  ramaddress := 0
  PrintStringCR(string("smile64"))  
  fat.openfile(string("smile64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                           ' 4096 * 2 bytes per pixel
    fat.readdata(@sdbuffer,512) 
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress 
    ramaddress := ramaddress + 512                        ' increment the counter
  fat.closefile
  ' now copy to the screen
  repeat x from 0 to 2
    repeat y from 0 to 4
      ILI9325.draw(x*64,y*64,(x+1)*64-1,(y+1)*64-1)                             ' set up the area of the screen to draw in 
      ILI9325.docmd("I",0,0,8192)                         ' move 4096 pixels (8192 bytes) from address 0 to the display           

      

PUB MemoryToDisplay
    ILI9325.draw(0,0,63,63)                             ' set up the area of the screen to draw in  
    ILI9325.docmd("I",0,0,8192)                         ' move 4096 pixels (8192 bytes) from address 0 to the display
    ILI9325.draw(64,0,127,63)                           ' set up the area of the screen to draw in 
    ILI9325.docmd("I",0,8192,8192)                      ' move 4096 pixels (8192 bytes) from address 0 to the display  
    ILI9325.draw(128,0,191,63)                           ' set up the area of the screen to draw in 
    ILI9325.docmd("I",0,16384,8192)                      ' move 4096 pixels (8192 bytes) from address 0 to the display  
    ILI9325.draw(0,64,63,127)                             ' set up the area of the screen to draw in  
    ILI9325.docmd("I",0,24576,8192)                         ' move 4096 pixels (8192 bytes) from address 0 to the display
    ILI9325.draw(64,64,127,127)                           ' set up the area of the screen to draw in 
    ILI9325.docmd("I",0,32768,8192)                      ' move 4096 pixels (8192 bytes) from address 0 to the display  
    ILI9325.draw(128,64,191,127)                           ' set up the area of the screen to draw in 
    ILI9325.docmd("I",0,40960,8192)                      ' move 4096 pixels (8192 bytes) from address 0 to the display  




'PUB MemoryTest | i                                          ' try writing and reading back a value to the ram
'  i := 100
'  ILI9325.docmd("W",@i,0,1)                                 ' write to ram
'  i := 101                                              ' change the value of i
'  ILI9325.docmd("R",@i,0,1)                                 ' read back from external ram
'  printdecimal(i)                                       ' should print 100

PUB IconstoMemory | ramaddress
  ramaddress := 0
  PrintStringCR(string("calc64"))  
  fat.openfile(string("calc64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                             ' 4096 * 2 bytes per pixel    
    fat.readdata(@sdbuffer,512) 
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress 
    ramaddress := ramaddress + 512                        ' increment the counter
  fat.closefile
  
  ramaddress := 8192
  PrintStringCR(string("clock64"))  
  fat.openfile(string("clock64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                           ' 4096 * 2 bytes per pixel / 512 per block
    fat.readdata(@sdbuffer,512)
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress
    ramaddress := ramaddress + 512                     ' increment the counter
  fat.closefile

  ramaddress := 16384
  PrintStringCR(string("smile64"))  
  fat.openfile(string("smile64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                           ' 4096 * 2 bytes per pixel
    fat.readdata(@sdbuffer,512) 
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress 
    ramaddress := ramaddress + 512                        ' increment the counter
  fat.closefile

  ramaddress := 24576
  PrintStringCR(string("disk64"))  
  fat.openfile(string("disk64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                          ' 4096 * 2 bytes per pixel
    fat.readdata(@sdbuffer,512) 
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress 
    ramaddress := ramaddress + 512                        ' increment the counter
  fat.closefile
  
  ramaddress := 32768
  PrintStringCR(string("game64"))  
  fat.openfile(string("game64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                          ' 4096 * 2 bytes per pixel
    fat.readdata(@sdbuffer,512) 
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress 
    ramaddress := ramaddress + 512                        ' increment the counter
  fat.closefile

  ramaddress := 40960
  PrintStringCR(string("prop64"))  
  fat.openfile(string("prop64.ili"),"R")               ' open an image (.ili extension) 
  repeat 16                                          ' 4096 * 2 bytes per pixel
    fat.readdata(@sdbuffer,512) 
    ILI9325.docmd("W",@sdbuffer,ramaddress,512)        ' write n to ramaddress 
    ramaddress := ramaddress + 512                        ' increment the counter
  fat.closefile

  PrintStringCR(string("touch the screen")) 

PRI TouchScreen  | yval, xval,x,y
' *** note these pins are changing in the next board to DO = 20, CLK =21, DI = 22, CS = 23 (busy and penirq not connected)
' also with penirq on pin 18 this upsets the TV display
'  Touch_Clock = 23                                       ' clock
'  Touch_ChipSelect = 22                                  ' chip select needs to be low to enable the touchscreen
'  Touch_DataIn = 21                                      ' data into the chip = an output from the prop
'  Touch_Busy = 20                                        ' busy ? not needed - just returns /cs
'  Touch_DataOut = 19                                     ' data out of the chip = an input to the prop
'  Touch_Penirq = 18                                      ' pen down (? use for wakeup)
  OUTA[Touch_ChipSelect] := 0                           ' enable the touch screen
  SPI.start(3,0)                                        ' delay,state
  'Shiftout = data,clock,mode,bits,value
  'shiftin = data,clock,mode,bits
  'SPI read S, A2, A1, A0, Mode, SER, PD1, PD0
  '         1  0   0   1    1     0    0    0
  ' MSB first
  ' 001 is X
  ' 101 is Y
    repeat
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1101_0000)  ' reads x from  500 to 3700  (off < 500 )
      xval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      SPI.SHIFTOUT(Touch_DataIn, Touch_Clock, 5, 8 , %1001_0000)  ' reads y from 400 to 3800 (off > 3800 ) 
      yval := SPI.SHIFTIN(Touch_DataOut,Touch_Clock,2, 12)
      if xval <500                                      ' if less than 500 then not touched, change to -1
        xval := -1                                      ' no touch is -1
      else
        xval := 3700 - xval                             ' move origin bottom right to bottom left
      if yval > 3800                                   ' if >3800 then not touched, change to -1
        yval := -1  
      delay.pause1ms(100)                              ' short delay for the display
      'printdecimal(xval)                               ' display the numbers
      'printdecimal(yval)
      'crlf                                             ' new line
      if (xval <> -1)  and (yval <> -1)
        x := (xval - 500)/800                           ' 0,1,2,3
        y := (yval - 400) / 680                         ' 0,1,2,3,4
        printdecimal(x)                               ' display the numbers
        printdecimal(y)
        crlf
        if (x == 0) and (y == 4)
          ILI9325.draw(0,0,63,63)                           ' set up the area of the screen to draw in  
          ILI9325.docmd("I",0,0,8192)
        if (x == 1) and (y == 4)
          ILI9325.draw(64,0,127,63)                           ' set up the area of the screen to draw in  
          ILI9325.docmd("I",0,8192,8192)
        if (x == 2) and (y == 4)  
          ILI9325.draw(128,0,191,63)                           ' set up the area of the screen to draw in  
          ILI9325.docmd("I",0,16384,8192)
        if (x == 0) and (y == 3)
          ILI9325.draw(0,64,63,127)                           ' set up the area of the screen to draw in  
          ILI9325.docmd("I",0,24576,8192)
        if (x == 1) and (y == 3)  
          ILI9325.draw(64,64,127,127)                           ' set up the area of the screen to draw in  
          ILI9325.docmd("I",0,32768,8192)
        if (x == 2) and (y == 3)
          ILI9325.draw(128,64,191,127)                           ' set up the area of the screen to draw in  
          ILI9325.docmd("I",0,40960,8192)

        delay.pause1ms(500)' only display once so delay half a second to "debounce" 
 

PUB PrintStringCR(pstring)' sends pstring to the vga and serial port
     PrintString(pstring)                               ' print the string  
     crlf

PUB PrintString(pstring)' sends pstring to the vga and serial port with no CRLF
    if (_TV_Enabled)
      text.str(pstring)
'    if (_VGA_Enabled)  
'      vt100.str(pstring)                                 ' send to vga screen
'    if (_Serial_Enabled)   
'    sio.str(0,pstring)                                 ' send to com port
'    if (_lcd_Enabled)
'       lcd.str(pstring)                                   ' send to 20x4 lCD display

PUB PrintChar(c) ' sends c to vga and serial port
    if (_TV_Enabled)
      text.out(c)
'    if (_VGA_Enabled)
'      vt100.out(c)                                        ' send to vga
'    if (_Serial_Enabled)
'      sio.tx(0,c)                                         ' send to com port
'    if (_lcd_Enabled)
'      LCD.out(c)                                          ' send to LCD display      
PUB CRLF
    PrintChar(13)                                        ' send to vga
    PrintChar(10)                                        ' send to vga

PUB PrintDecimal(number)                                ' print the decimal value, useful for debuggin
     PrintString(str.integerToDecimal(number, 10))

DAT
sdbuffer        byte    $0[512]  ' 512 byte buffer for sd card interface 